@@ -9,7 +9,6 @@ Email: nbowman@stanford.edu
 Date of Creation: 10/1/2019
 Last Modified: 3/31/2020
 """
-
 import importlib.util
 import inspect
 import os
@@ -31,17 +30,22 @@ class StudentCode:
     """
 
     def __init__(self, code_file, karel=None):
-        assert os.path.isfile(code_file), "{} could not be found.".format(code_file)
+        if not os.path.isfile(code_file):
+            raise FileNotFoundError(f"{code_file} could not be found.")
+
         self.module_name = os.path.basename(code_file)
         if self.module_name.endswith(".py"):
             self.module_name = os.path.splitext(self.module_name)[0]
-        spec = importlib.util.spec_from_file_location(self.module_name, os.path.abspath(code_file))
+
+        spec = importlib.util.spec_from_file_location(
+            self.module_name, os.path.abspath(code_file)
+        )
         try:
             self.mod = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(self.mod)
         except Exception as e:
             # Handle syntax errors and only print location of error
-            print("Syntax Error: {}".format(e))
+            print(f"Syntax Error: {e}")
             print("\n".join(tb.format_exc(limit=0).split("\n")[1:]))
             sys.exit()
 
@@ -50,16 +54,19 @@ class StudentCode:
             print("Couldn't find the main() function. Are you sure you have one?")
             sys.exit()
 
-        if karel:
+        if karel is not None:
             self.inject_namespace(karel)
 
+    def __repr__(self):
+        return inspect.getsource(self.mod)
+
     def inject_namespace(self, karel):
         """
-		This function is responsible for doing some Python hackery
-		that associates the generic commands the student wrote in their
-		file with specific commands relating to the Karel object that exists
-		in the world.
-		"""
+        This function is responsible for doing some Python hackery
+        that associates the generic commands the student wrote in their
+        file with specific commands relating to the Karel object that exists
+        in the world.
+        """
         functions_to_override = [
             "move",
             "turn_left",
@@ -109,7 +116,7 @@ class KarelApplication(tk.Frame):
         master.columnconfigure(1, weight=1)
 
         # set master geometry
-        master.geometry(str(window_width) + "x" + str(window_height))
+        master.geometry(f"{window_width}x{window_height}")
 
         super().__init__(master, background=LIGHT_GREY)
 
@@ -138,8 +145,11 @@ class KarelApplication(tk.Frame):
     def set_dock_icon(self):
         # make Karel dock icon image
         path = os.path.join(os.path.dirname(__file__), "icon.png")
-        img = tk.Image("photo", file=path)
-        self.master.tk.call("wm", "iconphoto", self.master._w, img)
+        try:
+            img = tk.Image("photo", file=path)
+            self.master.tk.call("wm", "iconphoto", self.master._w, img)
+        except Exception:
+            print(f"Warning: invalid icon.png: {path}")
 
     def create_menubar(self):
         menubar = tk.Menu(self.master)
@@ -166,10 +176,10 @@ class KarelApplication(tk.Frame):
 
     def create_slider(self):
         """
-		This method creates a frame containing three widgets:
-		two labels on either side of a scale slider to control
-		Karel execution speed.
-		"""
+        This method creates a frame containing three widgets:
+        two labels on either side of a scale slider to control
+        Karel execution speed.
+        """
         self.slider_frame = tk.Frame(self, bg=LIGHT_GREY)
         self.slider_frame.grid(row=3, column=0, padx=PAD_X, pady=PAD_Y, sticky="ew")
 
@@ -188,39 +198,44 @@ class KarelApplication(tk.Frame):
         self.scale.pack()
 
     def create_canvas(self):
-        """
-		This method creates the canvas on which Karel and Karel's
-		world are drawn.
-		"""
+        """ This method creates the canvas on which Karel and the world are drawn. """
         self.canvas = KarelCanvas(
-            self.canvas_width, self.canvas_height, self.master, world=self.world, karel=self.karel
+            self.canvas_width,
+            self.canvas_height,
+            self.master,
+            world=self.world,
+            karel=self.karel,
         )
         self.canvas.grid(column=1, row=0, sticky="NESW")
         self.canvas.bind("<Configure>", lambda t: self.canvas.redraw_all())
 
     def create_buttons(self):
         """
-		This method creates the three buttons that appear on the left
-		side of the screen. These buttons control the start of Karel
-		execution, resetting Karel's state, and loading new worlds.
-		"""
+        This method creates the three buttons that appear on the left
+        side of the screen. These buttons control the start of Karel
+        execution, resetting Karel's state, and loading new worlds.
+        """
         self.program_control_button = tk.Button(
             self, highlightthickness=0, highlightbackground="white"
         )
         self.program_control_button["text"] = "Run Program"
         self.program_control_button["command"] = self.run_program
-        self.program_control_button.grid(column=0, row=0, padx=PAD_X, pady=PAD_Y, sticky="ew")
+        self.program_control_button.grid(
+            column=0, row=0, padx=PAD_X, pady=PAD_Y, sticky="ew"
+        )
 
         self.load_world_button = tk.Button(
             self, highlightthickness=0, text="Load World", command=self.load_world
         )
-        self.load_world_button.grid(column=0, row=2, padx=PAD_X, pady=PAD_Y, sticky="ew")
+        self.load_world_button.grid(
+            column=0, row=2, padx=PAD_X, pady=PAD_Y, sticky="ew"
+        )
 
     def create_status_label(self):
-        """
-		This function creates the status label at the bottom of the window.
-		"""
-        self.status_label = tk.Label(self.master, text="Welcome to Karel!", bg=LIGHT_GREY)
+        """ This function creates the status label at the bottom of the window. """
+        self.status_label = tk.Label(
+            self.master, text="Welcome to Karel!", bg=LIGHT_GREY
+        )
         self.status_label.grid(row=1, column=0, columnspan=2)
 
     def karel_action_decorator(self, karel_fn):
@@ -261,16 +276,24 @@ class KarelApplication(tk.Frame):
 
     def inject_decorator_namespace(self):
         """
-		This function is responsible for doing some Python hackery
-		that associates the generic commands the student wrote in their
-		file with specific commands relating to the Karel object that exists
-		in the world.
-		"""
-        self.student_code.mod.turn_left = self.karel_action_decorator(self.karel.turn_left)
+        This function is responsible for doing some Python hackery
+        that associates the generic commands the student wrote in their
+        file with specific commands relating to the Karel object that exists
+        in the world.
+        """
+        self.student_code.mod.turn_left = self.karel_action_decorator(
+            self.karel.turn_left
+        )
         self.student_code.mod.move = self.karel_action_decorator(self.karel.move)
-        self.student_code.mod.pick_beeper = self.beeper_action_decorator(self.karel.pick_beeper)
-        self.student_code.mod.put_beeper = self.beeper_action_decorator(self.karel.put_beeper)
-        self.student_code.mod.paint_corner = self.corner_action_decorator(self.karel.paint_corner)
+        self.student_code.mod.pick_beeper = self.beeper_action_decorator(
+            self.karel.pick_beeper
+        )
+        self.student_code.mod.put_beeper = self.beeper_action_decorator(
+            self.karel.put_beeper
+        )
+        self.student_code.mod.paint_corner = self.corner_action_decorator(
+            self.karel.paint_corner
+        )
 
     def disable_buttons(self):
         self.program_control_button.configure(state="disabled")
@@ -292,8 +315,10 @@ class KarelApplication(tk.Frame):
             if self.student_code.module_name + ".py" in filename:
                 display_frames.append((frame, lineno))
 
-        print(("".join(tb.format_list(tb.StackSummary.extract(display_frames)))).strip())
-        print("{}: {}".format(type(e).__name__, str(e)))
+        print(
+            ("".join(tb.format_list(tb.StackSummary.extract(display_frames)))).strip()
+        )
+        print(f"{type(e).__name__}: {e}")
 
     def run_program(self):
         # Error checking for existence of main function completed in prior file
@@ -310,7 +335,9 @@ class KarelApplication(tk.Frame):
             )
             self.display_error_traceback(e)
             self.update()
-            showwarning("Karel Error", "Karel Crashed!\nCheck the terminal for more details.")
+            showwarning(
+                "Karel Error", "Karel Crashed!\nCheck the terminal for more details."
+            )
 
         finally:
             # Update program control button to force user
@@ -346,7 +373,7 @@ class KarelApplication(tk.Frame):
         # Reset speed slider
         self.scale.set(self.world.init_speed)
         self.status_label.configure(
-            text="Loaded world from {}.".format(os.path.basename(filename)), fg="black"
+            text=f"Loaded world from {os.path.basename(filename)}.", fg="black"
         )
 
         # Make sure program control button is set to 'run' mode
