@@ -12,16 +12,18 @@ Email: nbowman@stanford.edu
 Date of Creation: 10/1/2019
 Last Modified: 3/31/2020
 """
-import math
 from enum import Enum, unique
 
 
 @unique
 class Direction(Enum):
-    EAST = 0
-    SOUTH = math.pi / 2
-    WEST = math.pi
-    NORTH = 3 * math.pi / 2
+    EAST = "east"
+    SOUTH = "south"
+    WEST = "west"
+    NORTH = "north"
+
+    def __repr__(self) -> str:
+        return str(self.value)
 
 
 # Karel Application + World Editor
@@ -50,64 +52,39 @@ COLOR_MAP = {
     "Yellow": "yellow",
 }
 
-# Karel World, World Editor
-DIRECTIONS_MAP = {
-    "north": Direction.NORTH,
-    "east": Direction.EAST,
-    "south": Direction.SOUTH,
-    "west": Direction.WEST,
-}
-DIRECTIONS_MAP_INVERSE = {v: k for k, v in DIRECTIONS_MAP.items()}
-
 
 # Karel World, Karel Canvas
 class Wall:
     """ Note that the World Editor only uses West & South to denote wall directions. """
 
     def __init__(self, avenue, street, direction):
-        self._avenue = avenue
-        self._street = street
-        self._direction = direction
+        self.avenue = avenue
+        self.street = street
+        self.direction = direction
 
     def __eq__(self, other):
         return self.__dict__ == other.__dict__
 
     def __hash__(self):
-        return hash((self._avenue, self._street, self._direction))
+        return hash((self.avenue, self.street, self.direction))
 
     def __repr__(self):
-        return "({}, {}) {}".format(self._avenue, self._street, self._direction)
-
-    @property
-    def avenue(self):
-        return self._avenue
-
-    @property
-    def street(self):
-        return self._street
-
-    @property
-    def direction(self):
-        return self._direction
+        return f"Wall({self.avenue}, {self.street}, {self.direction})"
 
 
 # Karel Application + Karel
 class KarelException(Exception):
-    """
-	The following classes define Karel-specific exceptions.
-	"""
+    """ The following classes define Karel-specific exceptions. """
 
     def __init__(self, avenue, street, direction, message):
         super().__init__()
         self.avenue = avenue
         self.street = street
-        self.direction = DIRECTIONS_MAP_INVERSE[direction].capitalize()
+        self.direction = direction.value.capitalize()
         self.message = message
 
     def __str__(self):
         return (
-            "Karel crashed while on avenue {} and street {}, "
-            "facing {}\nInvalid action: {}".format(
-                self.avenue, self.street, self.direction, self.message
-            )
+            f"Karel crashed while on avenue {self.avenue} and street {self.street}, "
+            f"facing {self.direction}\nInvalid action: {self.message}"
         )
